BiquadCascade

Struct BiquadCascade 

pub struct BiquadCascade<T, const N: usize> {
    pub sections: [Biquad<T>; N],
}
A cascade of N biquad (second-order) sections.

Filter order is 2*N for all-second-order sections, or 2*N - 1 when the last section is a degenerate first-order biquad (b2 = a2 = 0).

Example

use numeris::control::{butterworth_lowpass, BiquadCascade};

// 4th-order Butterworth → 2 biquad sections
let mut lpf: BiquadCascade<f64, 2> = butterworth_lowpass(4, 1000.0, 8000.0).unwrap();
let y = lpf.tick(1.0);

Fields

sections: [Biquad<T>; N]

Implementations

Source

impl<T: FloatScalar, const N: usize> BiquadCascade<T, N>

pub fn tick(&mut self, x: T) -> T

Process a single input sample through all sections in series.

pub fn reset(&mut self)

Reset all sections’ internal state to zero.

pub fn process(&mut self, input: &[T], output: &mut [T])

Process a slice of input samples into an output slice.

Panics

Panics if output.len() < input.len().

pub fn process_inplace(&mut self, data: &mut [T])

Process a slice of samples in-place.

pub fn order(&self) -> usize

Actual filter order (detects degenerate first-order last section).