pub struct BiquadCascade<T, const N: usize> {
pub sections: [Biquad<T>; N],
}Expand description
A cascade of N biquad (second-order) sections.
Filter order is 2*N for all-second-order sections, or 2*N - 1 when the
last section is a degenerate first-order biquad (b2 = a2 = 0).
Example
use numeris::control::{butterworth_lowpass, BiquadCascade};
// 4th-order Butterworth → 2 biquad sections
let mut lpf: BiquadCascade<f64, 2> = butterworth_lowpass(4, 1000.0, 8000.0).unwrap();
let y = lpf.tick(1.0);Fields
sections: [Biquad<T>; N]Implementations
Sourceimpl<T: FloatScalar, const N: usize> BiquadCascade<T, N>
impl<T: FloatScalar, const N: usize> BiquadCascade<T, N>
Sourcepub fn tick(&mut self, x: T) -> T
pub fn tick(&mut self, x: T) -> T
Process a single input sample through all sections in series.
Sourcepub fn process(&mut self, input: &[T], output: &mut [T])
pub fn process(&mut self, input: &[T], output: &mut [T])
Process a slice of input samples into an output slice.
Panics
Panics if output.len() < input.len().
Sourcepub fn process_inplace(&mut self, data: &mut [T])
pub fn process_inplace(&mut self, data: &mut [T])
Process a slice of samples in-place.